@@ -14,6 +14,7 @@
 
 
 import collections
+import copy
 import glob
 import os
 import re
@@ -42,7 +43,8 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
     """Connector class to attach/detach iSCSI volumes."""
 
     supported_transports = ['be2iscsi', 'bnx2i', 'cxgb3i', 'default',
-                            'cxgb4i', 'qla4xxx', 'ocs', 'iser']
+                            'cxgb4i', 'qla4xxx', 'ocs', 'iser', 'tcp']
+    VALID_SESSIONS_PREFIX = ('tcp:', 'iser:')
 
     def __init__(self, root_helper, driver=None,
                  execute=None, use_multipath=False,
@@ -266,7 +268,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         """Check that given iscsi_iface uses only supported transports
 
         Accepted transport names for provided iface param are
-        be2iscsi, bnx2i, cxgb3i, cxgb4i, default, qla4xxx, ocs or iser.
+        be2iscsi, bnx2i, cxgb3i, cxgb4i, default, qla4xxx, ocs, iser or tcp.
         Note the difference between transport and iface;
         unlike default(iscsi_tcp)/iser, this is not one and the same for
         offloaded transports, where the default format is
@@ -411,6 +413,8 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
                               {'target_portal': connection_properties[
                                   'target_portal']})
                     raise
+            old_node_startups = self._get_node_startup_values(
+                connection_properties)
             out = self._run_iscsiadm_bare(
                 ['-m', 'discoverydb',
                  '-t', 'sendtargets',
@@ -418,13 +422,19 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
                  '-p', connection_properties['target_portal'],
                  '--discover'],
                 check_exit_code=[0, 255])[0] or ""
+            self._recover_node_startup_values(connection_properties,
+                                              old_node_startups)
         else:
+            old_node_startups = self._get_node_startup_values(
+                connection_properties)
             out = self._run_iscsiadm_bare(
                 ['-m', 'discovery',
                  '-t', 'sendtargets',
                  '-I', iscsi_transport,
                  '-p', connection_properties['target_portal']],
                 check_exit_code=[0, 255])[0] or ""
+            self._recover_node_startup_values(connection_properties,
+                                              old_node_startups)
 
         ips, iqns = self._get_target_portals_from_iscsiadm_output(out)
         luns = self._get_luns(connection_properties, iqns)
@@ -782,7 +792,8 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         nodes = self._get_iscsi_nodes()
         sessions = self._get_iscsi_sessions_full()
         # Use (portal, iqn) to map the session value
-        sessions_map = {(s[2], s[4]): s[1] for s in sessions if s[0] == 'tcp:'}
+        sessions_map = {(s[2], s[4]): s[1] for s in sessions
+                        if s[0] in self.VALID_SESSIONS_PREFIX}
         # device_map will keep a tuple with devices from the connection and
         # others that don't belong to this connection" (belong, others)
         device_map = collections.defaultdict(lambda: (set(), set()))
@@ -973,7 +984,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         iqns = []
         for data in [line.split() for line in output.splitlines()]:
             if len(data) == 2 and data[1].startswith('iqn.'):
-                ips.append(data[0])
+                ips.append(data[0].split(',')[0])
                 iqns.append(data[1])
         return ips, iqns
 
@@ -1020,7 +1031,8 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
             sessions = self._get_iscsi_sessions_full()
             for s in sessions:
                 # Found our session, return session_id
-                if 'tcp:' == s[0] and portal == s[2] and s[4] == target_iqn:
+                if (s[0] in self.VALID_SESSIONS_PREFIX and
+                        portal == s[2] and s[4] == target_iqn):
                     return s[1], manual_scan
 
             try:
@@ -1034,7 +1046,6 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
                             {'iqn': target_iqn, 'portal': portal,
                              'err': err.exit_code})
                 return None, None
-
             self._iscsiadm_update(connection_properties,
                                   "node.startup",
                                   "automatic")
@@ -1089,3 +1100,47 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
                   {'multipath_command': multipath_command,
                    'out': out, 'err': err})
         return (out, err)
+
+    def _get_node_startup_values(self, connection_properties):
+        out, __ = self._run_iscsiadm_bare(
+            ['-m', 'node', '--op', 'show', '-p',
+             connection_properties['target_portal']]) or ""
+        node_values = out.strip()
+        node_values = node_values.split("\n")
+        iqn = None
+        startup = None
+        startup_values = {}
+
+        for node_value in node_values:
+            node_keys = node_value.split()
+            try:
+                if node_keys[0] == "node.name":
+                    iqn = node_keys[2]
+                elif node_keys[0] == "node.startup":
+                    startup = node_keys[2]
+
+                if iqn and startup:
+                    startup_values[iqn] = startup
+                    iqn = None
+                    startup = None
+            except IndexError:
+                pass
+
+        return startup_values
+
+    def _recover_node_startup_values(self, connection_properties,
+                                     old_node_startups):
+        node_startups = self._get_node_startup_values(connection_properties)
+        for iqn, node_startup in node_startups.items():
+            old_node_startup = old_node_startups.get(iqn, None)
+            if old_node_startup and node_startup != old_node_startup:
+                # _iscsiadm_update() only uses "target_portal" and "target_iqn"
+                # of connection_properties.
+                # And the recovering target belongs to the same target_portal
+                # as discovering target.
+                # So target_iqn is updated, and other values aren't updated.
+                recover_connection = copy.deepcopy(connection_properties)
+                recover_connection['target_iqn'] = [iqn]
+                self._iscsiadm_update(recover_connection,
+                                      "node.startup",
+                                      old_node_startup)
